package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.CoreFactory;
import org.openxma.dsl.core.model.BooleanParameterValue;
import org.openxma.dsl.core.model.BooleanValue;
import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.IDocumentable;
import org.openxma.dsl.core.model.IntegerParameterValue;
import org.openxma.dsl.core.model.ParameterDefinition;
import org.openxma.dsl.core.model.ParameterValue;
import org.openxma.dsl.core.model.ReferenceWithParameter;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.StringParameterValue;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.core.model.Type;
import org.openxma.dsl.core.model.Validator;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.core.util.Literals;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.helper.style.ApplyAlignmentStyle;
import org.openxma.dsl.generator.helper.style.ApplyCombinedStyles;
import org.openxma.dsl.pom.PomPackage;
import org.openxma.dsl.pom.model.Button;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.ContentAlignment;
import org.openxma.dsl.pom.model.ControlType;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.DataObjectVariable;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.IField;
import org.openxma.dsl.pom.model.Image;
import org.openxma.dsl.pom.model.IncludePanel;
import org.openxma.dsl.pom.model.Label;
import org.openxma.dsl.pom.model.LabelText;
import org.openxma.dsl.pom.model.Orientation;
import org.openxma.dsl.pom.model.Seperator;
import org.openxma.dsl.pom.model.StylePropertyBorder;
import org.openxma.dsl.pom.model.StylePropertyEnumerationDisplayType;
import org.openxma.dsl.pom.model.StylePropertyEnumerationSortOrder;
import org.openxma.dsl.pom.model.StylePropertyImage;
import org.openxma.dsl.pom.model.StylePropertyLineWrap;
import org.openxma.dsl.pom.model.StylePropertyMultiLine;
import org.openxma.dsl.pom.model.StylePropertyScrollable;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabableFlag;
import org.openxma.dsl.pom.model.TableColumn;
import org.openxma.dsl.pom.model.Text;
import org.openxma.dsl.pom.model.Tree;
import org.openxma.dsl.pom.naming.PomNameProvider;
import org.openxma.dsl.pom.util.StyleAccess;
import org.openxma.dsl.pom.util.TextProperties;

import at.spardat.xma.guidesign.BDAttribute;
import at.spardat.xma.guidesign.BDCollection;
import at.spardat.xma.guidesign.CheckButton;
import at.spardat.xma.guidesign.DataLabel;
import at.spardat.xma.guidesign.DatePicker;
import at.spardat.xma.guidesign.GuidesignFactory;
import at.spardat.xma.guidesign.PushButton;
import at.spardat.xma.guidesign.SimpleCombo;
import at.spardat.xma.guidesign.XMAButton;
import at.spardat.xma.guidesign.XMACombo;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMAContainer;
import at.spardat.xma.guidesign.XMAFormData;
import at.spardat.xma.guidesign.XMALabel;
import at.spardat.xma.guidesign.XMAList;
import at.spardat.xma.guidesign.XMAScrolledComposite;
import at.spardat.xma.guidesign.XMASeperator;
import at.spardat.xma.guidesign.XMAText;
import at.spardat.xma.guidesign.XMATree;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.CollapseChain;
import at.spardat.xma.guidesign.flex.IXMADocumentable;
import at.spardat.xma.guidesign.types.AlignmentType;
import at.spardat.xma.guidesign.types.DomainDisplayType;
import at.spardat.xma.guidesign.types.DomainFormatType;
import at.spardat.xma.guidesign.types.DomainOrderType;
import at.spardat.xma.guidesign.types.SeperatorType;

public class WidgetExtension {
    
    private static final String GENERATOR_FLAG_POSTFIX = "_generatorFlag";    
    private static final String GUI_ELEMENT_PROPERTY_NAME = "guiElement";
    private static final String FIELD_PART_PROPERTY_NAME = "fieldPart";
    private static final String FIELD_PART_SPECIFICATION_PROPERTY_NAME = "fieldPartSpecification";
    private static final String SOURCE_MODEL_ELEMENT_PROPERTY_NAME = "sourceModelElement";

    public static void setDefaultWidgetProperties(XMAComponent xmaComp) {

        if (xmaComp != null) {
            try {
                setAllWidgetPropertiesForEmptyStyle(xmaComp);
                setWidgetPropertiesForDefinedStyles(xmaComp);
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new RuntimeException("Default properties can not be set",ex);
            }
        }
    }
    
    public static void setJavaDocComments(XMAComponent xmaComp) {
        setJavaDocComment(xmaComp);
        if (xmaComp != null) {
            TreeIterator<EObject> allContents = xmaComp.eAllContents();
            while (allContents.hasNext()) {
                EObject obj = allContents.next();
                setJavaDocComment(obj);
            }
        }
    }

    private static void setJavaDocComment(EObject obj) {
        if (obj instanceof IXMADocumentable) {
            IXMADocumentable xmaDocumentableElement = (IXMADocumentable) obj;
            EObject sourceObject = getSourceModelElementProperty(obj);
            if (sourceObject instanceof IDocumentable) {
                IDocumentable documentableElement = (IDocumentable) sourceObject;
                if (documentableElement.getTxtDescription() != null)
                    xmaDocumentableElement.setTxtDescription(documentableElement.getTxtDescription());
            }
        }
    }

    private static void setAllWidgetPropertiesForEmptyStyle(EObject xmaComp) {

        TreeIterator<EObject> allContents = xmaComp.eAllContents();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (obj instanceof XMAWidget) {
                XMAWidget xmaWidget = (XMAWidget)obj;
                GuiElement guiElement = getGuiElementProperty(xmaWidget);
                setWidgetProperties(xmaWidget,guiElement,null);
            }
        }
    }


    private static void setWidgetPropertiesForDefinedStyles(EObject modelElement) {

        // Apply own style
        Iterator<EObject> allContents = modelElement.eContents().iterator();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (hasNoChildrenWhichAreWidgets(obj)) {
                continue;
            }
            
            EObject sourceModelElement = getSourceModelElementProperty(obj);
            if (sourceModelElement != null) {
                /* New style implementation */
                Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(sourceModelElement);
                if (styleProperties != null) {
                    ApplyCombinedStyles.applyToGuidesignModelObject(obj,styleProperties);
                }
                /* Old style implementation */
                // TODO: refactore to new style implementation (see above)
                if (obj instanceof XMAWidget) {
                    XMAWidget xmaWidget = (XMAWidget)obj; 
                    if (sourceModelElement instanceof GuiElement) {
                        GuiElement guiElement = (GuiElement)sourceModelElement;
                        List<Style> styles = PresentationModelExtension.getStyleProperty(guiElement);
                        if (styles != null) {
                            for (Style style : styles) {
                                setWidgetProperties(xmaWidget,guiElement,style);
                                setChildrenWidgetProperties(xmaWidget,style);
                            }
                        }                        
                    }                    
                }                
            }                     
            setWidgetPropertiesForDefinedStyles(obj);
        }
    }

    private static void setChildrenWidgetProperties(EObject modelElement, Style parentStyle) {
        TreeIterator<EObject> allContents = modelElement.eAllContents();
        while (allContents.hasNext()) {
            EObject obj = allContents.next();
            if (obj instanceof XMAWidget) {
                XMAWidget xmaWidget = (XMAWidget)obj;
                GuiElement guiElement = getGuiElementProperty(xmaWidget);
                setWidgetProperties(xmaWidget,guiElement,parentStyle);
            }
        }
    }

    private static void setWidgetProperties(XMAWidget widget, GuiElement guiElement, Style style) {
        if (style != null && style instanceof StyleSpecification && (StylePropertyExtension.fitsStyleSpecification(guiElement,(StyleSpecification)style))) {
            StyleSpecification sSpec = (StyleSpecification)style;
            for (StyleSpecificationProperty sp : sSpec.getStyleProperty()) {
                if (sp instanceof StylePropertyBorder) {
                    StylePropertyBorder spb = (StylePropertyBorder)sp;
                    if (spb.isNoneBorder()) {
                        widget.setYnBorder(false);
                    } else if (spb.isSolid()) {
                        // In case of a scrollable-Content the border is only applied to the ScrolledComposite
                        if (widget.eContainer() instanceof XMAScrolledComposite == false) {
                            widget.setYnBorder(true);
                        }
                    }
                } else if (sp instanceof StylePropertyImage) {
                    StylePropertyImage spi = (StylePropertyImage)sp;
                    if (widget instanceof XMAButton) {
                        XMAButton but = (XMAButton)widget;
                        but.setUriImage(spi.getImageUri());
                    } else if (widget instanceof XMALabel) {
                        XMALabel lab = (XMALabel)widget;
                        lab.setUriImage(spi.getImageUri());
                    }
                }
//                else if (sp instanceof StylePropertyAlignment) {
//                    StylePropertyAlignment spa = (StylePropertyAlignment)sp;
//                     if (widget instanceof XMAText) {
//                         XMAText text = (XMAText)widget;
//                         text.setCodAlignment(contentAlignment2AlignmentType(spa.getAlignment()));
//                     } else if (widget instanceof XMALabel) {
//                         XMALabel label = (XMALabel)widget;
//                         label.setCodAlignment(contentAlignment2AlignmentType(spa.getAlignment()));
//                     } else if (widget instanceof DataLabel) {
//                         DataLabel label = (DataLabel)widget;
//                         label.setCodAlignment(contentAlignment2AlignmentType(spa.getAlignment()));
//                     }
//                }
            }
        }
        String styleName = (style != null && style.getName() != null) ? style.getName():null;
        GeneratorConfiguration.INSTANCE.getLayoutStrategy().setXMAWidgetProperties(widget, guiElement, styleName);
    }

    private static boolean hasNoChildrenWhichAreWidgets(EObject modelElement) {
        if (modelElement instanceof XMAFormData) {
            return true;
        }
        if (modelElement instanceof BDCollection) {
            return true;
        }
        if (modelElement instanceof CollapseChain) {
            return true;
        }
        return false;
    }
    
    public static boolean getGeneratorFlag(EObject widget, String flagName) {
        Object value = EObjectPropertiesAdapter.getProperty(widget,flagName+GENERATOR_FLAG_POSTFIX);
        if (value instanceof Boolean) {
            return (Boolean)value;
        }
        return false;
    } 
    
    public static void setGeneratorFlag(EObject widget, String flagName, Boolean value) {
        EObjectPropertiesAdapter.setProperty(widget,flagName+GENERATOR_FLAG_POSTFIX,value);            
    }      

    public static XMAWidget setGuiElementProperty(XMAWidget widget, EObject obj) {
        if (obj instanceof GuiElement) {
            GuiElement guiElement = (GuiElement)obj;
            EObjectPropertiesAdapter.setProperty(widget,GUI_ELEMENT_PROPERTY_NAME,guiElement);
        }
        return widget;
    }

    public static GuiElement getGuiElementProperty(XMAWidget widget) {
        Object value = EObjectPropertiesAdapter.getProperty(widget,GUI_ELEMENT_PROPERTY_NAME);
        if (value instanceof GuiElement || value == null) {
            return (GuiElement)value;
        }
        throw new RuntimeException("The property '"+GUI_ELEMENT_PROPERTY_NAME+"' is not of type 'org.openxma.xmadsl.model.GuiElement'");
    }

    public static void setSourceModelElementProperty(EObject widget, EObject obj) {
        if (obj != null) {
            EObjectPropertiesAdapter.setProperty(widget,SOURCE_MODEL_ELEMENT_PROPERTY_NAME,obj);
        }
    }

    public static EObject getSourceModelElementProperty(EObject widget) {
        EObject sourceModelElement = (EObject)EObjectPropertiesAdapter.getProperty(widget,SOURCE_MODEL_ELEMENT_PROPERTY_NAME); 
        if (sourceModelElement == null && widget instanceof XMAWidget) {        
            sourceModelElement = getGuiElementProperty((XMAWidget)widget);
        }
        return sourceModelElement;                
    }

    public static XMAWidget setFieldPartSpecificationProperty(XMAWidget widget, FieldPartSpecification obj) {
        if (obj != null) {
            EObjectPropertiesAdapter.setProperty(widget,FIELD_PART_SPECIFICATION_PROPERTY_NAME,obj);
        }
        return widget;
    }

    public static XMAWidget setFieldPartProperty(XMAWidget widget, FieldPart obj) {
        if (obj != null) {
            EObjectPropertiesAdapter.setProperty(widget,FIELD_PART_PROPERTY_NAME,obj);
        }
        return widget;
    }

    public static FieldPart getFieldPartProperty(XMAWidget widget) {
        Object value = EObjectPropertiesAdapter.getProperty(widget,FIELD_PART_PROPERTY_NAME);
        if (value instanceof FieldPart || value == null) {
            return (FieldPart)value;
        }
        throw new RuntimeException("The property '"+FIELD_PART_PROPERTY_NAME+"' is not of type 'org.openxma.xmadsl.model.FieldPart'");
    }

    public static FieldPartSpecification getFieldPartSpecificationProperty(XMAWidget widget) {
        Object value = EObjectPropertiesAdapter.getProperty(widget,FIELD_PART_SPECIFICATION_PROPERTY_NAME);
        if (value instanceof FieldPartSpecification || value == null) {
            return (FieldPartSpecification)value;
        }
        throw new RuntimeException("The property '"+FIELD_PART_SPECIFICATION_PROPERTY_NAME+"' is not of type 'org.openxma.xmadsl.model.FieldPart'");
    }

    public static List<XMAWidget> createWidgetsForGuiElement(Seperator s, XMAComposite xmaComp, List businessDataList) {
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
        XMASeperator seperator = GuidesignFactory.eINSTANCE.createXMASeperator();
        
        setTabable(seperator,s);           

        String name = s.getName();

        if (s.getOrientation() != null && s.getOrientation().getValue() == Orientation.VERTICAL_VALUE) {
            seperator.setCodSerperator(SeperatorType.VERTICAL_LITERAL);
        } else {
            seperator.setCodSerperator(SeperatorType.HORIZONTAL_LITERAL);
        }

        seperator.setYnGenerated(true);
        seperator.setNamInstance(name);

        xmaComp.getControls().add(seperator);
        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(seperator,s);              
        setGuiElementProperty(seperator,s);

        widgetList.add(seperator);
        return widgetList;
    }

    public static List<XMAWidget> createWidgetsForGuiElement(Text text, XMAComposite xmaComp, List businessDataList) {

        XMAWidget newText = createXMAText(text);
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();

        if (text.getTextProperties() != null) {
            String labelText = TextProperties.getLabelText(text.getTextProperties());
            if (labelText != null) {
                //IWidgetWithLabel widgetWithLabel = (IWidgetWithLabel)newText;
                XMALabel newLabel = createLabel(text,text.getName(),null,false);
                Util.setLabelText(newLabel,labelText+":");
                xmaComp.getControls().add(newLabel);
                widgetList.add(newLabel);
                setFieldPartProperty(newLabel,FieldPart.LABEL);
                setFieldPartProperty(newText,FieldPart.CONTROL);
            }
        }

        if (newText instanceof XMAText) {
            Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(text);
            XMAText xmaText = (XMAText)newText;
            Boolean isMultiLine = Literals.booleanForBoolLiteral(text.getMultiLine());
            if (isMultiLine != null) {
                xmaText.setYnMulitRow(isMultiLine);
            } else {
                StylePropertyMultiLine multiLineStyle = PresentationModelExtension.getMultiLineStyle(styleProperties);
                if (multiLineStyle != null && multiLineStyle.getMulti() instanceof TrueLiteral) {
                    xmaText.setYnMulitRow(true);
                }
            }
            StylePropertyLineWrap lineWrapStyle = PresentationModelExtension.getLineWrapStyle(styleProperties);
            if (lineWrapStyle != null && lineWrapStyle.getWrap() instanceof TrueLiteral) {
                xmaText.setYnWrap(true);
            }


            ApplyAlignmentStyle.applyAlignment(xmaText, text.getAlignment(),styleProperties);
            setScrollableFlags(styleProperties,xmaText);
        }

        xmaComp.getControls().add(newText);
        widgetList.add(newText);
        return widgetList;
    }



    private static void setScrollableFlags(Map<Integer,StyleSpecificationProperty> styleSpecificationProperties, XMAText xmaText) {
        if (styleSpecificationProperties != null) {
            StylePropertyScrollable scrollableProp = (StylePropertyScrollable)styleSpecificationProperties.get(PomPackage.STYLE_PROPERTY_SCROLLABLE);
            if (scrollableProp != null) {
                if (scrollableProp.isNotScrollable()) {
                    xmaText.setYnHScroll(false);
                    xmaText.setYnVScroll(false);
                } else {
                    for (Orientation orientation : scrollableProp.getOrientations()) {
                        if (orientation.getValue() == Orientation.HORIZONTAL_VALUE) {
                            xmaText.setYnHScroll(true);
                        } else {
                            xmaText.setYnVScroll(true);
                        }
                    }
                }
            }
        }
    }

    public static AlignmentType contentAlignment2AlignmentType(ContentAlignment contentAlignment) {
        if (contentAlignment == null) return null;
        switch (contentAlignment.getValue()) {
        case ContentAlignment.LEFT_VALUE:
            return AlignmentType.LEFT_LITERAL;
        case ContentAlignment.RIGHT_VALUE:
            return AlignmentType.RIGHT_LITERAL;
        case ContentAlignment.CENTER_VALUE:
            return AlignmentType.CENTER_LITERAL;
        }
        throw new RuntimeException("Unknown ContentAlignment: "+contentAlignment);
    }        

    public static List<XMAWidget> createWidgetsForGuiElement(Button b, XMAComposite xmaComp, List businessDataList) {

        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
        PushButton pushButton = GuidesignFactory.eINSTANCE.createPushButton();

        // pushButton.ynSelectionEvent = false
        //EAttribute attribute = GuidesignPackage.Literals.PUSH_BUTTON__YN_SELECTION_EVENT;
        //pushButton.eSet(attribute, true);

        pushButton.setYnGenerated(true);
        pushButton.setNamInstance(b.getName());
        pushButton.setLabelName(b.getLabelText().getLabel());

        if (b.getImageUri() != null) {
            pushButton.setUriImage(b.getImageUri());
        } else if (b.getImageUriReference() != null) {
            pushButton.setUriImage(b.getImageUriReference().getSource());
        }

        Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(b);
        ApplyAlignmentStyle.applyAlignment(pushButton, b.getAlignment(),styleProperties);

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(pushButton,b);        
        xmaComp.getControls().add(pushButton);
        setGuiElementProperty(pushButton,b);

        widgetList.add(pushButton);
        return widgetList;
    }
    
    public static List<XMAWidget> createWidgetsForGuiElement(CheckBox b, XMAComposite xmaComp, List businessDataList) {
        
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();        
        CheckButton newButton = GuidesignFactory.eINSTANCE.createCheckButton();

        newButton.setYnGenerated(true);
        newButton.setNamInstance(b.getName());
        newButton.setLabelName(getLabelString(b.getLabelText()));
        xmaComp.getControls().add(newButton);

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(newButton,b);                
        widgetList.add(newButton);            
        return widgetList;
    }
    
    public static List<XMAWidget> createWidgetsForGuiElement(org.openxma.dsl.pom.model.RadioButton b, XMAComposite xmaComp, List businessDataList) {
        
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
        at.spardat.xma.guidesign.RadioButton newButton = GuidesignFactory.eINSTANCE.createRadioButton();        

        newButton.setYnGenerated(true);
        newButton.setNamInstance(b.getName());
        newButton.setLabelName(getLabelString(b.getLabelText()));
        xmaComp.getControls().add(newButton);

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(newButton,b);                                
        widgetList.add(newButton);            
        return widgetList;
    }    
    
    public static List<XMAWidget> createWidgetsForGuiElement(Tree t, XMAComposite xmaComp, List businessDataList) {
                
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
        XMATree newTree = GuidesignFactory.eINSTANCE.createXMATree();
        
        setTabable(newTree,t);          

        newTree.setYnGenerated(true);
        newTree.setNamInstance(t.getName());
        xmaComp.getControls().add(newTree);

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(newTree,t);                                           
        widgetList.add(newTree);            
        return widgetList;
    }
    
    public static List<XMAWidget> createWidgetsForGuiElement(Label t, XMAComposite xmaComp, List businessDataList) {        
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();               
        XMALabel newLabel = createXMALabel(t); 
        setTabable(newLabel,t);                  
        xmaComp.getControls().add(newLabel);
        widgetList.add(newLabel);        
        return widgetList;
    }    
    
    public static List<XMAWidget> createWidgetsForGuiElement(Image t, XMAComposite xmaComp, List businessDataList) {        
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();        
        XMAWidget newImage = createXMAImage(t);
        setTabable(newImage,t);        
        xmaComp.getControls().add(newImage);
        widgetList.add(newImage);        
        return widgetList;
    }        
    
    public static List<XMAWidget> createWidgetsForGuiElement(IncludePanel includePanel, XMAComposite xmaComp, List businessDataList) {

        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
        XMAContainer newContainer = GuidesignFactory.eINSTANCE.createXMAContainer();
        
        setTabable(newContainer,includePanel);          
             
        newContainer.setYnGenerated(true);
        newContainer.setNamInstance(includePanel.getName());
        xmaComp.getControls().add(newContainer);
        if (includePanel.getXmaComponent() != null) {
            newContainer.setUriComponent(includePanel.getXmaComponent().getNamClass());
        } else if (includePanel.getComponent() != null) {
            newContainer.setUriComponent(includePanel.getComponent().getXmaComponent().getNamClass());            
        } else if (includePanel.getEmbeddedPage() != null) {
            newContainer.setEmbeddedPage(includePanel.getEmbeddedPage());
        }
            
        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(newContainer,includePanel);                 
        widgetList.add(newContainer);            
        return widgetList;
    }          

    public static List<XMAWidget> createWidgetsForGuiElement(Combo combo, XMAComposite xmaComp, List businessDataList) {
        
        XMAWidget widget = null;
        List<XMAWidget> widgetList = new ArrayList<XMAWidget>();        
        
        // Create domain combo in case an according formatter is set. Otherwise create a simple combo. 
        if (combo.getFormat() != null) {
            String baseValidatorNameOfField = BasicTypeHelper.getBaseValidatorName(combo.getFormat());
            if ("I18NEnumerationValidator".equals(baseValidatorNameOfField)) {
                widget = createDomain(combo);
            } else {
                widget = createXMASimpleCombo(combo);                
            }
        } else if (combo.getType() != null) {
            widget = createDomain(combo,combo.getType());                    
        } else {
            widget = createXMASimpleCombo(combo);    
        }
        
        if (combo.getTextProperties() != null) {
            String labelText = TextProperties.getLabelText(combo.getTextProperties());
            if (labelText != null) {
                XMALabel newLabel = createLabel(combo,combo.getName(),null,false);
                Util.setLabelText(newLabel,labelText+":");
                xmaComp.getControls().add(newLabel);
                widgetList.add(newLabel);
                setFieldPartProperty(newLabel,FieldPart.LABEL);
                setFieldPartProperty(widget,FieldPart.CONTROL);
            }
        }        

        xmaComp.getControls().add(widget);
        widgetList.add(widget);
        return widgetList;
    }


    private static List<FieldPart> getFieldPartsList(CustomizeableField f) {
        List<FieldPart> parts = new ArrayList<FieldPart>();
        for (FieldPartSpecification fps : f.getParts()) {
            parts.add(fps.getPart());
        }
        return parts;
    }

    /**
     *
     * @param f
     * @param businessDataList
     * @param xmaComp
     * @return
     */
    public static List<XMAWidget> createWidgetsForGuiElement(CustomizeableField f, XMAComposite xmaComp, List businessDataList) {

        try {
            List<XMAWidget> widgetList = new ArrayList<XMAWidget>();
            BDAttribute bdAttribute = Util.findBDAttributeForFeature(xmaComp.getComponent(),businessDataList,f.getFeature(),f.getObject());
            String defaultWidgetName = PomNameProvider.composeDefaultWidgetNameForField(f);
            
            Attribute attr = f.getAttribute();
            DataTypeAndTypeParameter type = attr.getType();            
            WidgetType widgetType = BasicTypeHelper.determineWidgetType(f);
            
            boolean isCheckBoxLeftAligned = GeneratorConfiguration.INSTANCE.getLayoutStrategy().isCheckBoxLeftAligned();            

            int indexLabel = 0;
            int indexControl = 1;
            int indexUnit = 2;
            int indexEditor = 3;
            int maxPartIndex = 3;
            List<FieldPart> parts = getFieldPartsList(f);
            if (parts != null && parts.size()>0) {
                indexLabel = parts.indexOf(FieldPart.LABEL);
                indexControl = parts.indexOf(FieldPart.CONTROL);
                indexUnit = parts.indexOf(FieldPart.UNITLABEL);
                indexEditor = parts.indexOf(FieldPart.EDITOR);
                maxPartIndex = parts.size()-1;
                // In case that the order of label and control for a checkbox
                // is explicitly defined, the defined order overrides the 
                // generator properties
                if (widgetType == WidgetType.CHECKBOX) {
                    if (indexControl == 1 && indexLabel == 0) {
                        isCheckBoxLeftAligned = false;
                    }
                }                                
            }

            // Create the XMAWidgets for the field
            XMAWidget dataWidget = indexControl > -1 ? createControl(f,widgetType,type,isCheckBoxLeftAligned) : null;
            XMALabel labelWidget = indexLabel > -1 ? createLabel(f,defaultWidgetName,type,isCheckBoxLeftAligned) : null;
            XMAWidget unitWidget = indexUnit > -1 ? createUnitWidget(xmaComp,f,defaultWidgetName,businessDataList) : null;
            XMAWidget customEditorWidget = indexEditor > -1 ? SimpleTypeExtension.createCustomEditorWidget(f,defaultWidgetName,xmaComp) : null;

            int partSpecificationSize = f.getParts().size();
            for (int i=0;i <= maxPartIndex;i++) {
                if (i == indexLabel && labelWidget != null) {
                    xmaComp.getControls().add(labelWidget);
                    String colon = indexControl == -1 || indexControl == indexLabel+1 ? ":" : "";
                    Util.setLabelText(labelWidget,f.getLabelText()+colon);
                    if (dataWidget != null && dataWidget instanceof CheckButton == false) {
                        Util.attachWidgetAndLabel(dataWidget,labelWidget);
                    }
                    if (indexLabel < partSpecificationSize) {
                        setFieldPartSpecificationProperty(labelWidget,f.getParts().get(indexLabel));
                    }
                    setFieldPartProperty(labelWidget,FieldPart.LABEL);
                    widgetList.add(labelWidget);
                }
                if (i == indexControl && dataWidget != null) {
                    xmaComp.getControls().add(dataWidget);
                    Util.setTooltipText(dataWidget,f.getTooltipText());
                    Util.setBDAttribute(dataWidget,bdAttribute);
                    //ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(dataWidget,f);
                    if (indexControl < partSpecificationSize) {
                        setFieldPartSpecificationProperty(dataWidget,f.getParts().get(indexControl));
                    }
                    setFieldPartProperty(dataWidget,FieldPart.CONTROL);
                    widgetList.add(dataWidget);
                }
                if (i == indexUnit && unitWidget != null) {
                    xmaComp.getControls().add(unitWidget);
                    if (indexUnit < partSpecificationSize) {
                        setFieldPartSpecificationProperty(unitWidget,f.getParts().get(indexUnit));
                    }
                    setFieldPartProperty(unitWidget,FieldPart.UNITLABEL);
                    widgetList.add(unitWidget);
                }
                if (i == indexEditor && customEditorWidget != null) {
                    xmaComp.getControls().add(customEditorWidget);
                    if (indexEditor < partSpecificationSize) {
                        setFieldPartSpecificationProperty(customEditorWidget,f.getParts().get(indexEditor));
                    }
                    setFieldPartProperty(customEditorWidget,FieldPart.EDITOR);
                    widgetList.add(customEditorWidget);
                }
            }

            return widgetList;

        } catch (Exception ex) {
            ex.printStackTrace();
            throw new RuntimeException("Can not create XMAWidgets for field",ex);
        }
    }

    /*
     * Create a data control according the standard layout
     */
    private static XMAWidget createControl(CustomizeableField f, WidgetType widgetType, DataTypeAndTypeParameter type, boolean isCheckBoxLeftAligned) {
                
        //WidgetType widgetType = BasicTypeHelper.determineWidgetType(f);
        switch(widgetType) {
        case DOMAINCOMBO:
            return createDomain(f,type);
        case DATALABEL:
            return createDataLabel(f);
        case SIMPLECOMBO:
            return createSimpleCombo(f,type);
        case LISTBOX:
            return createListBox(f,type);
        case TEXT:
            return createText(f);
        case CHECKBOX:
            return createCheckBox(f,isCheckBoxLeftAligned);
        case DATEPICKER:
            return createDatePicker(f);
        default:
            throw new RuntimeException("Can not create control for field: "+f);
        }

// TODO: delete the following commented lines
//        // Determine control type from validator of field
//        if (f.getFormat() != null) {
//            String baseValidatorNameOfField = BasicTypeHelper.getBaseValidatorName(f.getFormat());
//            if ("I18NEnumerationValidator".equals(baseValidatorNameOfField)) {
//                return createDomain(f,type);
//            }
//        }
//
//        // Determine control type from explicit control-type-specification of field
//        if (f.getControlType() != null) {
//            if (BasicTypeHelper.useStaticDataLabel(f)) {
//                return createDataLabel(f);
//            } else if (f.getControlType().equals(ControlType.COMBO)) {
//                return createSimpleCombo(f,type);
//            } else if (f.getControlType().equals(ControlType.LISTBOX)) {
//                return createListBox(f,type);
//            } else if (f.getControlType().equals(ControlType.TEXT)) {
//                return createText(f);
//            }
//        }
//
//        // Determine control type implicit from type
//        if (BasicTypeHelper.isI18NEnumeration(type)) {
//            return createDomain(f,type);
//        }
//        if (BasicTypeHelper.isBoolean(type)) {
//            return createCheckBox(f);
//        }
//        if (BasicTypeHelper.isDate(type)) {
//            return createDatePicker(f);
//        }
//        return createText(f);
    }
    
    /**
     * Create ParameterValueCollector from a simple ValidatorReference
     * 
     * @param validatorReference
     * @return
     */
    private static ParameterValueCollector createParameterValueCollector(ValidatorReference validatorReference) {
        ParameterValueCollector paramCollector = new ParameterValueCollector();
        paramCollector.addParameterValues(getParameterValueRecord(validatorReference));        
        return paramCollector;
    }
    
    /**
     * Create ParameterValueCollector from a simple DataTypeAndTypeParameter
     * 
     * @param dataTypeAndTypeParameter
     * @return
     */
    private static ParameterValueCollector createParameterValueCollector(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ParameterValueCollector paramCollector = new ParameterValueCollector();
        paramCollector.addParameterValues(getParameterValueRecord(dataTypeAndTypeParameter));        
        return paramCollector;
    }    

    /**
     * Create ParameterValueCollector from a IField (e.g. CustomizeableField) determining the parameter values
     * from the formatter of the field, the formatter of the attribute and the valiator of the data type.
     * 
     * @param f
     * @param dataTypeAndTypeParameter
     * @return
     */
    private static ParameterValueCollector createParameterValueCollector(IField f, DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ParameterValueCollector paramCollector = new ParameterValueCollector();
        if (f instanceof CustomizeableField && ((CustomizeableField)f).getFormat() != null) {
            paramCollector.addParameterValues(getParameterValueRecord(((CustomizeableField)f).getFormat()));
        }
        if (paramCollector.isMoreRequired()) {
            ValidatorReference fmt = f.getAttributeHolder().getFormat();
            if (fmt != null) {
                paramCollector.addParameterValues(getParameterValueRecord(fmt));
            }
        }
        if (paramCollector.isMoreRequired()) {
            paramCollector.addParameterValues(getParameterValueRecord(dataTypeAndTypeParameter));
        }
        return paramCollector;
    }
    
    private static void setDataSource(XMACombo xmaCombo, String dataSource) {
        if (dataSource != null) {
            Util.setDataSource(xmaCombo,dataSource);
        } else {
            xmaCombo.setYnDynamic(true);
        }        
    }
    
    private static XMAWidget createDomain(Combo combo) {
        
        XMACombo xmaCombo = GuidesignFactory.eINSTANCE.createXMACombo();
        xmaCombo.setNamInstance(combo.getName());
        xmaCombo.setYnBorder(true);
        xmaCombo.setYnGenerated(true);
        
        setTabable(xmaCombo,combo);             
        
        ValidatorReference validatorReference = combo.getFormat();
        ParameterValueCollector paramCollector = createParameterValueCollector(validatorReference);
        
        /* Set dataSource */
        String dataSource = getXMADomainSpezification(paramCollector);        
        setDataSource(xmaCombo,dataSource);
        /* Set additional properties */        
        setAdditionalXMAComboProperties(combo, paramCollector, xmaCombo);
        
        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(xmaCombo,combo);
        setGuiElementProperty(xmaCombo,combo);
        return xmaCombo;        
    }
    
    /*
     * Create a combo according the standard layout
     */
    private static XMAWidget createDomain(Combo f, DataTypeAndTypeParameter dataTypeAndTypeParameter) {

        ParameterValueCollector paramCollector = createParameterValueCollector(dataTypeAndTypeParameter);

        XMACombo combo = GuidesignFactory.eINSTANCE.createXMACombo();
        combo.setNamInstance(f.getName());
        combo.setYnBorder(true);
        combo.setYnGenerated(true);
        
        setTabable(combo,f);                

        /* Set dataSource */
        Type type = dataTypeAndTypeParameter.getDataType();
        SimpleType simpleType = (type instanceof SimpleType) ? (SimpleType)type : null;
        String dataSource = getXMADomainSpezification(simpleType,paramCollector);
        setDataSource(combo,dataSource);
        /* Set additional properties */
        setAdditionalXMAComboProperties(f, paramCollector, combo);

        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(combo,(CustomizeableField)f);
        }            
        setGuiElementProperty(combo,f);
        return combo;
    }    

    /*
     * Create a combo according the standard layout
     */
    private static XMAWidget createDomain(IField f, DataTypeAndTypeParameter dataTypeAndTypeParameter) {

        ParameterValueCollector paramCollector = createParameterValueCollector(f,dataTypeAndTypeParameter);

        XMACombo combo = GuidesignFactory.eINSTANCE.createXMACombo();
        combo.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        combo.setYnBorder(true);
        combo.setYnGenerated(true);
        
        setTabableForIField(combo,f,true);                

        /* Set dataSource */
        Type type = dataTypeAndTypeParameter.getDataType();
        SimpleType simpleType = (type instanceof SimpleType) ? (SimpleType)type : null;
        String dataSource = getXMADomainSpezification(simpleType,paramCollector);
        setDataSource(combo,dataSource);
        /* Set additional properties */
        setAdditionalXMAComboProperties(f, paramCollector, combo);

        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(combo,(CustomizeableField)f);
        }            
        setGuiElementProperty(combo,f);
        return combo;
    }

    private static void setAdditionalXMAComboProperties(EObject comboElement, ParameterValueCollector paramCollector, XMACombo combo) {
        Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(comboElement);
        // DisplayType from validator (TODO_ )
        String displayTypeFromValidator = paramCollector.getParameterValueString(2);
        if ("short".equals(displayTypeFromValidator)) {
            combo.setCodDisplaytype(DomainDisplayType.SHOW_SHORT_LITERAL);
        } else if (("short_long").equals(displayTypeFromValidator)) {
            combo.setCodDisplaytype(DomainDisplayType.SHOW_SHORT_LONG_LITERAL);
        } else if (("long").equals(displayTypeFromValidator)) {
            combo.setCodDisplaytype(DomainDisplayType.SHOW_LONG_LITERAL);
        }
        // DisplayType from style
        StylePropertyEnumerationDisplayType displayTypeProp = PresentationModelExtension.getEnumerationDisplayTypeStyle(styleProperties);
        if (displayTypeProp != null) {
            combo.setCodDisplaytype(mapToDomainDisplayType(displayTypeProp));
        }

        // SortOrder from validator
        String sortOrderFromValidator = paramCollector.getParameterValueString(3);
        if ("short".equals(sortOrderFromValidator)) {
            combo.setCodOrder(DomainOrderType.ORDER_SHORT_LITERAL);
        } else if (("natural").equals(sortOrderFromValidator)) {
            combo.setCodOrder(DomainOrderType.ORDER_NATURAL_LITERAL);
        } else if (("long").equals(sortOrderFromValidator)) {
            combo.setCodOrder(DomainOrderType.ORDER_LONG_LITERAL);
        }
        // SortOrder from style
        StylePropertyEnumerationSortOrder sortOrderProp = PresentationModelExtension.getEnumerationSortOrderStyle(styleProperties);
        if (sortOrderProp != null) {
            String sortOrder = sortOrderProp.getSortorder();
            if ("short".equals(sortOrder)) {
                combo.setCodOrder(DomainOrderType.ORDER_SHORT_LITERAL);
            } else if (("natural").equals(sortOrder)) {
                combo.setCodOrder(DomainOrderType.ORDER_NATURAL_LITERAL);
            } else if (("long").equals(sortOrder)) {
                combo.setCodOrder(DomainOrderType.ORDER_LONG_LITERAL);
            }
        }
    }
    
    public static DomainDisplayType mapToDomainDisplayType(StylePropertyEnumerationDisplayType displayTypeProp) {
        if (displayTypeProp != null) {
            String displayType = displayTypeProp.getType();
            if ("short".equals(displayType)) {
                return DomainDisplayType.SHOW_SHORT_LITERAL;
            } else if (("short_long").equals(displayType)) {
                return DomainDisplayType.SHOW_SHORT_LONG_LITERAL;
            } else if (("long").equals(displayType)) {
                return DomainDisplayType.SHOW_LONG_LITERAL;
            }
        }
        return null;
    }
    
    public static DomainFormatType mapToDomainFormatType(StylePropertyEnumerationDisplayType displayTypeProp) {
        if (displayTypeProp != null) {
            String displayType = displayTypeProp.getType();
            if ("short".equals(displayType)) {
                return DomainFormatType.SHORT_LITERAL;
            } else if (("short_long").equals(displayType)) {
                return DomainFormatType.SHORT_LONG_LITERAL;
            } else if (("long").equals(displayType)) {
                return DomainFormatType.LONG_LITERAL;
            }
            // DomainFormatType "key" not supported
        }
        return null;
    }    

    /*
     * Create a combo according the standard layout
     */
    private static XMAWidget createSimpleCombo(IField f, DataTypeAndTypeParameter dataTypeAndTypeParameter) {

        SimpleCombo combo = GuidesignFactory.eINSTANCE.createSimpleCombo();

        combo.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        combo.setYnBorder(true);
        combo.setYnGenerated(true);
        
        setTabableForIField(combo,f,true);                

        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(combo,(CustomizeableField)f);
        }            
        setGuiElementProperty(combo,f);
        return combo;
    }

    private static XMAWidget createListBox(IField f, DataTypeAndTypeParameter dataTypeAndTypeParameter) {

        XMAList combo = GuidesignFactory.eINSTANCE.createXMAList();

        combo.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        combo.setYnBorder(true);
        combo.setYnGenerated(true);
        
        setTabableForIField(combo,f,true);                

        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(combo,(CustomizeableField)f);
        }            
        setGuiElementProperty(combo,f);
        return combo;
    }

    /*
     * Create a text field according the standard layout
     */
    private static XMAWidget createDatePicker(IField f) {

        DatePicker text = GuidesignFactory.eINSTANCE.createDatePicker();

        text.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        text.setYnBorder(true);
        text.setYnGenerated(true);
        
        setTabableForIField(text,f,true);               
        
        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(text,(CustomizeableField)f);
        }        
        setGuiElementProperty(text,f);
        return text;
    }

    /*
     * Create a text field according the standard layout
     */
    private static XMAWidget createCheckBox(CustomizeableField f, boolean isCheckBoxLeftAligned) {

        CheckButton checkButton = GuidesignFactory.eINSTANCE.createCheckButton();

        checkButton.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        checkButton.setYnGenerated(true);
        
        setTabableForIField(checkButton,f,true);        
        
        if (isCheckBoxLeftAligned) {
            checkButton.setCodAlignment(AlignmentType.LEFT_LITERAL);
            checkButton.setLabelName(f.getLabelText());
        }  
        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(checkButton,(CustomizeableField)f);
        }            
        setGuiElementProperty(checkButton,f);
        return checkButton;
    }

    public static EditableFlag editableFlagOfWidget(IElementWithLayoutData elementWithLayoutData) {
        if (elementWithLayoutData == null) {
            return null;
        }
        if (elementWithLayoutData.getFieldFlags() != null) {
            for (FieldFlag ff : elementWithLayoutData.getFieldFlags()) {
                if (ff instanceof EditableFlag) {
                    return (EditableFlag)ff;
                }
            }
        }
        return null;
    }
    
    public static void setTabable(XMAWidget xmaWidget, IElementWithLayoutData elementWithLayoutData) {
        TabableFlag tf  = elementWithLayoutData.getTabableFlag();
        if (tf != null) {
            if (tf.getValue() instanceof TrueLiteral) {
                xmaWidget.setYnTabSequence(true);
            } else if (tf.getValue() instanceof FalseLiteral) {
                xmaWidget.setYnTabSequence(false);
            }                    
        }
    }
    
    public static void setTabableForIField(XMAWidget xmaWidget, IField iField, boolean defaultValue) {        
        if (iField instanceof CustomizeableField) {
            setTabable(xmaWidget,(CustomizeableField)iField);
        } else {
            xmaWidget.setYnTabSequence(defaultValue);
        }                                      
    }

    /**
     * Create a DataLabel for the IField f
     *
     * @param f
     * @return
     */
    private static XMAWidget createDataLabel(IField f) {

        Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(f);

        DataLabel dataLabel = GuidesignFactory.eINSTANCE.createDataLabel();
        dataLabel.setYnBorder(false);
        StylePropertyLineWrap lineWrapStyle = PresentationModelExtension.getLineWrapStyle(styleProperties);
        if (lineWrapStyle != null && lineWrapStyle.getWrap() instanceof TrueLiteral) {
            dataLabel.setYnWrap(true);
        }

        dataLabel.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        dataLabel.setYnGenerated(true);


        ApplyAlignmentStyle.applyAlignment(dataLabel,null,styleProperties);

        if (f instanceof CustomizeableField) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(dataLabel,(CustomizeableField)f);
        }            
        setGuiElementProperty(dataLabel,f);
        return dataLabel;
    }

    /*
     * Create a text field according the standard layout
     */
    private static XMAWidget createText(IField f) {
        CustomizeableField cf = null;
        if (f instanceof CustomizeableField) {
            cf = (CustomizeableField)f;
        }

        Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(cf);

        XMAText text = GuidesignFactory.eINSTANCE.createXMAText();
        text.setYnBorder(true);    
        
        setTabableForIField(text,f,true);
        
        if (cf != null) {                                 
            Boolean isMultiLine = Literals.booleanForBoolLiteral(cf.getMultiLine());
            if (isMultiLine != null) {
                text.setYnMulitRow(isMultiLine);
            } else {
                StylePropertyMultiLine multiLineStyle = PresentationModelExtension.getMultiLineStyle(styleProperties);
                if (multiLineStyle != null && multiLineStyle.getMulti() instanceof TrueLiteral) {
                    text.setYnMulitRow(true);
                }
            }
            StylePropertyLineWrap lineWrapStyle = PresentationModelExtension.getLineWrapStyle(styleProperties);
            if (lineWrapStyle != null && lineWrapStyle.getWrap() instanceof TrueLiteral) {
                text.setYnWrap(true);
            }
            setScrollableFlags(styleProperties,text);
        }

        text.setNamInstance(PomNameProvider.composeDefaultWidgetNameForField(f));
        text.setYnGenerated(true);

        ApplyAlignmentStyle.applyAlignment(text,null,styleProperties);
        if (cf != null) {
            ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(text,cf);
        }
        setGuiElementProperty(text,f);
        return text;
    }



    /**
     * Create a label control
     */
    static XMALabel createLabel(GuiElement g, String defaultWidgetName,DataTypeAndTypeParameter type, boolean isCheckBoxLeftAligned) {
        // Dont create a label for a Checkbox if it's style is left-aligned
        if (type != null && BasicTypeHelper.isBoolean(type) && isCheckBoxLeftAligned) {
            return null;
        }
        XMALabel label = GuidesignFactory.eINSTANCE.createXMALabel();
        label.setYnWrap(true);
        label.setNamInstance(defaultWidgetName + "L");
        label.setYnBorder(false);
        label.setYnTabSequence(false);
        label.setYnGenerated(true);   

        setGuiElementProperty(label,g);
        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(label,g);        
        return label;
    }

    public static XMAWidget createXMAText(Text text) {

        XMAText widget =  GuidesignFactory.eINSTANCE.createXMAText();

        widget.setYnGenerated(true);
        widget.setNamInstance(text.getName());         
        
        setTabable(widget,text);        

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(widget,text);

        return widget;
    }

    public static SimpleCombo createXMASimpleCombo(Combo combo) {

        String name = combo.getName();

        SimpleCombo xmaText =  GuidesignFactory.eINSTANCE.createSimpleCombo();

        xmaText.setYnGenerated(true);
        xmaText.setNamInstance(name);
        if (combo.getStrictFlag() != null) {
            if (combo.getStrictFlag().getStrict() instanceof FalseLiteral) {
                xmaText.setYnStrict(false);
            } else {
                xmaText.setYnStrict(true);
            }
        }
        
        setTabable(xmaText,combo);            

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(xmaText,combo);

        return xmaText;
    }

    public static XMALabel createXMALabel(Label label) {

        String name = label.getName();
        if (name == null) {
            name=PomNameProvider.composeDefaultNameForGuiElement(label);
        }

        XMALabel xmaLabel =  GuidesignFactory.eINSTANCE.createXMALabel();

        xmaLabel.setYnGenerated(true);
        xmaLabel.setNamInstance(name);

        String labelString = WidgetExtension.getLabelString(label.getLabelText());
        xmaLabel.setLabelName(labelString);

        if (label.getImageUri() != null) {
            xmaLabel.setUriImage(label.getImageUri());
        } else if (label.getImageUriReference() != null) {
            xmaLabel.setUriImage(label.getImageUriReference().getSource());
        }

        Map<Integer,StyleSpecificationProperty> styleProperties = StyleAccess.getCombinedStyleProperty(label);
        ApplyAlignmentStyle.applyAlignment(xmaLabel, label.getAlignment(),styleProperties);

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(xmaLabel,label);

        return xmaLabel;
    }

    public static XMAWidget createXMAImage(Image image) {

        String name = image.getName();
        if (name == null) {
            name=PomNameProvider.composeDefaultNameForGuiElement(image);
        }

        XMALabel xmaLabel =  GuidesignFactory.eINSTANCE.createXMALabel();

        xmaLabel.setYnGenerated(true);
        xmaLabel.setNamInstance(name);

        if (image.getImageUri() != null) {
            xmaLabel.setUriImage(image.getImageUri());
        } else if (image.getImageUriReference() != null) {
            xmaLabel.setUriImage(image.getImageUriReference().getSource());
        } else {
            throw new RuntimeException("Image not found for '"+name+"'");
        }

        ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(xmaLabel,image);

        return xmaLabel;
    }

    /**
     * Create a XMAWidget which displays the unit for the data
     */
    private static XMAWidget createUnitWidget(XMAComposite xmaComp,CustomizeableField f, String defaultWidgetName, List businessDataList) {

        String unitText = f.getUnitText();
        Attribute unitAttribute = f.getUnitAttribute();
        DataObjectVariable dataObject = f.getObject();

        XMAWidget unitWidget = null;
        if (unitText != null) {
            unitWidget = createUnitLabel(defaultWidgetName,unitText);
        } else if (unitAttribute != null) {
            unitWidget = createUnitText(xmaComp,defaultWidgetName,unitAttribute,businessDataList,dataObject);
        }
        if (unitWidget != null) {
            if (f instanceof CustomizeableField) {
                ComplexTypeExtension.setLayoutDataPropertyFromGuiElement(unitWidget,(CustomizeableField)f);
            }                           
            setGuiElementProperty(unitWidget,f);
        }
        return unitWidget;
    }

    /*
     * Create a label-widget which displays the unit-string for the data
     */
    private static XMALabel createUnitLabel(String defaultWidgetName, String unitText) {

        XMALabel label = GuidesignFactory.eINSTANCE.createXMALabel();

        label.setYnWrap(true);
        label.setNamInstance(defaultWidgetName + "UL");
        label.setYnBorder(false);
        label.setYnTabSequence(false);
        label.setYnGenerated(true);

        Util.setLabelText(label,unitText);          

        return label;
    }

    private static XMAText createUnitText(XMAComposite xmaComp,String defaultWidgetName, Attribute unitAttribute, List businessDataList, DataObjectVariable dataObject) {

        XMAText text = GuidesignFactory.eINSTANCE.createXMAText();
        BDAttribute bdUnitAttribute = Util.findBDAttributeForAttribute(xmaComp.getComponent(),businessDataList,unitAttribute,dataObject);

        text.setYnWrap(false);
        text.setNamInstance(defaultWidgetName + "UL");
        text.setYnEditable(false);
        text.setYnBorder(false);
        text.setYnTabSequence(false);
        text.setYnGenerated(true);

        Util.setBDAttribute(text,bdUnitAttribute);

        return text;
    }

    public static ParameterValueRecord getParameterValueRecord(ValidatorReference validatorReferenceOfField) {
        List<ReferenceWithParameter> definitionStack = validatorReferenceOfField.getDefinitionStack();
        ValidatorReference validatorReference = validatorReferenceOfField.getRootValidatorReference();
        return new ParameterValueRecord(validatorReference.getParameterValues(),definitionStack);
    }

    public static ParameterValueRecord getParameterValueRecord(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        List<ReferenceWithParameter> definitionStack = dataTypeAndTypeParameter.getDefinitionStack();
        ValidatorReference validatorReference = dataTypeAndTypeParameter.getValidatorReferenceDefinition();
        if (validatorReference != null) {
            return new ParameterValueRecord(validatorReference.getParameterValues(),definitionStack);
        } else {
            return null;
        }
    }

    public static List<String> getValildatorParameterValues(ValidatorReference validatorReferenceOfField) {
        List<ReferenceWithParameter> definitionStack = validatorReferenceOfField.getDefinitionStack();
        ValidatorReference validatorReference = validatorReferenceOfField.getRootValidatorReference();
        return getValidatorParameterValuesInternal(definitionStack, validatorReference);
    }

    public static List<String> getValildatorParameterValues(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        List<ReferenceWithParameter> definitionStack = dataTypeAndTypeParameter.getDefinitionStack();
        ValidatorReference validatorReference = dataTypeAndTypeParameter.getValidatorReferenceDefinition();
        return getValidatorParameterValuesInternal(definitionStack, validatorReference);
    }

    private static List<String> getValidatorParameterValuesInternal(List<ReferenceWithParameter> definitionStack,ValidatorReference validatorReference) {
        List<String> parameterValues = new ArrayList<String>();
        List<ParameterValue> validatorParameters = validatorReference.getParameterValues();
        for (ParameterValue validatorParameter : validatorParameters) {
            String strValue = getParameterValue(validatorParameter,definitionStack,definitionStack.size()-1);
            parameterValues.add(strValue);
        }
        return parameterValues;
    }


    private static String getXMADomainSpezification(ParameterValueCollector paramCollector) {
        String dataSourcePrefix = paramCollector.getParameterValueString(0);
        String dataSourceName = paramCollector.getParameterValueString(1);
        String spezification = dataSourceName.length() != 0
            ? dataSourcePrefix+dataSourceName
            : null;

        return spezification;
    }    

    private static String getXMADomainSpezification(SimpleType simpleType, ParameterValueCollector paramCollector) {
        if (simpleType instanceof EnumType && paramCollector.getParameterCount()==0) {
            return getDefaultDomainSpecificationForEnum((EnumType)simpleType);            
        } else {
            return getXMADomainSpezification(paramCollector);
        }
    }
    
    public static String getDefaultDomainSpecificationForEnum(EnumType enumType) {
        return "type=rsc,bundle="+NameExtension.getPackageNameInternal(enumType,false)+"."+NameExtension.getName(enumType);
    }
    
    /**
     * Create a default ValidatorReference for a Enum. In this case the domain specification
     * is derived from the package and the name of the Enum.
     * 
     * It is assumed that there exists an according resource bundle. Therefore the
     * type is set to "rsc".
     */
    public static ValidatorReference getDefaultValidatorReferenceForEnum(EnumType enumType) {
        ValidatorReference validatorReference = CoreFactory.eINSTANCE.createValidatorReference();
        Validator validator = CoreFactory.eINSTANCE.createValidator();
        validator.setName(ValidatorReference.I18N_ENUMERATION_VALIDATOR);        
        validatorReference.setValidator(validator);
        StringParameterValue parameterValue = CoreFactory.eINSTANCE.createStringParameterValue();        
        parameterValue.setStringValue(getDefaultDomainSpecificationForEnum(enumType));
        validatorReference.getParameterValues().add(parameterValue);
        return validatorReference;        
    }

    public static String getParameterValue(ParameterValue pv, List<ReferenceWithParameter> definitionStack, int stackIndex) {

        if (pv.getReferencedParameter() == null) {
            return getValueAsString(pv);
        }
        return getValueOfReferencedParameter(pv.getReferencedParameter(),definitionStack,stackIndex);
    }

    private static String getValueAsString(ParameterValue pv) {
        if (pv == null) {
            return null;
        }
        if (pv instanceof IntegerParameterValue) {
            int intValue = ((IntegerParameterValue)pv).getIntValue();
            return Integer.toString(intValue);
        }
        if (pv instanceof StringParameterValue) {
            return ((StringParameterValue)pv).getStringValue();
        }
        if (pv instanceof BooleanParameterValue) {
            BooleanValue booleanValue = ((BooleanParameterValue)pv).getBooleanValue();
            return booleanValue.toString();
        }
        return null;
    }

    private static String getValueOfReferencedParameter(ParameterDefinition pd, List<ReferenceWithParameter> definitionStack, int stackIndex) {

        ReferenceWithParameter currentDefinition = definitionStack.get(stackIndex);

        List<ParameterValue> parameters = currentDefinition.getParameterValues();
        List<ParameterDefinition> parameterDefs = currentDefinition.getParameterDefinitions();
        String pdName = pd.getName();

        ParameterDefinition def = null;
        for (ParameterDefinition aDef : parameterDefs) {
            if (aDef.getName().equals(pdName)) {
                def = aDef;
                break;
            }
        }

        int index = parameterDefs.indexOf(def);
        if (def != null) {
            return getParameterValueForIndex(parameters,index,definitionStack,stackIndex-1);
        }
        return getValueOfReferencedParameter(pd,definitionStack,stackIndex-1);
    }

    private static String getParameterValueForIndex(List<ParameterValue> parameters, int index, List<ReferenceWithParameter> definitionStack, int stackIndex) {
        if (index > -1 && parameters.size() > index) {
            return getParameterValue(parameters.get(index),definitionStack,stackIndex);
        }
        return null;
    }

    public static String getValueForIndex(List<String>stringList, int index) {
        if (stringList.size() > index && stringList.get(index) != null) {
            return stringList.get(index);
        }
        return "";
    }

    /**
     * Compose the default table column name for a attribute
     */
    public static String composeDefaultTableColumnName(TableColumn column) {
        return PomNameProvider.composeDefaultWidgetNameForColumn(column);
    }




    public static String getLabelString(LabelText labelText) {
        if (labelText != null) {
            return labelText.getString();
        } else {
            return null;
        }
    }
}
